/*

Purpose:  DBA SQL Instance quick quality check 
Run this query in  SQL server Management Studio 
Feel free to pitch-in if you have any recommendations to modify this script.
*/ 

EXEC sp_configure 'xp_cmdshell' , 1 
PRINT 'Ignore the line above.' 
RECONFIGURE 
PRINT char ( 13 ) 
PRINT char ( 13 ) 
PRINT char ( 13 ) 

USE FTS_Mon 
GO 

SET NOCOUNT ON; 
DECLARE @errorlog varchar ( 500 ) 
PRINT '***************************************************************' 
PRINT '            DBA SQL Instance quality check ' 
PRINT '                                                               ' 
PRINT ' A. See SQL server name                          ' 
PRINT ' B. See Instance name              ' 
PRINT ' C. See Current Date Time          ' 
PRINT ' D. See SQL version      ' 
PRINT ' E. See ErrorLog file location               ' 
PRINT ' F. See per our standard Login audit mode we want is  2 = failed Logins only   ' 
PRINT ' G. See server auth, Mixed Mode = 0 or 2 ,Integrated = 1' 
PRINT ' H. See Name of Members in SysAdmin role' 
PRINT ' I. See Name of members in ServerAdmin' 
PRINT ' J. See Temp DB File Location.' 
PRINT ' K. See MDF, LDF & NDF File Location' 
PRINT ' L. See Max server Memory(GB)' 
PRINT ' M. See Min server memory (GB)' 
PRINT ' N. See Lock Pages in Memory' 
PRINT '***************************************************************' 

PRINT '' 
PRINT ' General Info' 
PRINT '****************************************************************' 
PRINT '' 

PRINT 'A. SQL Server Name.....................: ' + convert ( varchar ( 30 ) , @@SERVERNAME ) 
PRINT 'B. Instance............................: ' + convert ( varchar ( 30 ) , @@SERVICENAME ) 
PRINT 'C. Current Date Time...................: ' + convert ( varchar ( 30 ) , getdate ( ) , 113 ) 
PRINT 'D. SQL version.........................: ' + convert ( varchar ( 300 ) , @@VERSION ) 

SELECT @errorlog = replace ( cast ( serverproperty ( 'ErrorLogFileName' ) AS varchar ( 500 ) ) , 'ERRORLOG' , '' ) 

PRINT 'E. ErrorLog file location..............: ' + @errorlog 

DECLARE @AuditLevel int , 
@AuditLvltxt varchar ( 50 ) 

EXEC master.dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE' , 
N'Software\Microsoft\MSSQLServer\MSSQLServer' , N'AuditLevel' , @AuditLevel OUTPUT 

SELECT @AuditLvltxt = 
CASE 
    WHEN @AuditLevel = 0 THEN 'None.' 
    WHEN @AuditLevel = 1 THEN 'Successful logins only.' 
    WHEN @AuditLevel = 2 THEN 'Failed logins only.' 
    WHEN @AuditLevel = 3 THEN 'Both failed and successful logins.' 
    ELSE 'Unknown.' 
END 

--print 'F. AuditLevel.................: ' + convert(varchar(300), @AuditLevel)

PRINT 'F. AuditLevel..........................: ' + @AuditLvltxt 

DECLARE @LoginMode int , 
@LoginModetxt varchar ( 50 ) 

EXEC master.dbo.xp_instance_regread N'HKEY_LOCAL_MACHINE' , 
N'Software\Microsoft\MSSQLServer\MSSQLServer' , N'LoginMode' , @LoginMode OUTPUT 

SELECT @LoginModetxt = 
CASE 
    WHEN @LoginMode IN ( 0 , 2 ) THEN 'SQL Server and Windows Authentication mode.' 
    WHEN @LoginMode = 1 THEN 'Windows Authentication mode.' 
    ELSE 'Unknown.' 
END 

PRINT 'G. LoginMode...........................: ' + @LoginModetxt 
--select 'we want LoginMode = 1'

--Verify Account RoleMembers for serverAdmin and SYSAdmin

SET NOCOUNT ON 
PRINT 'H. Name of Members in SysAdmin role....:' 

--DECLARE @sysAdmin table (ServerRole CHAR(20), name CHAR(50), memberid VARBINARY(128))
DECLARE @sysAdmin TABLE ( ServerRole char ( 20 ) , name sysname ) 
DECLARE @ServerRole char ( 20 ) , @name char ( 50 ) 

--INSERT @sysAdmin exec sp_helpsrvrolemember 'sysadmin' Changed to IS_SRVROLEMEMBER('sysadmin', name), 
INSERT INTO @sysAdmin 
SELECT 
'sysadmin' , 
name COLLATE DATABASE_DEFAULT AS MemberName 
FROM sys.server_principals WHERE is_srvrolemember ( 'sysadmin' , name ) = 1 

DECLARE sysAdmin_cursor CURSOR FOR 
SELECT 
ServerRole , 
name 
FROM @sysAdmin 

OPEN sysAdmin_cursor 

FETCH NEXT FROM sysAdmin_cursor INTO 
@ServerRole , @name 

WHILE ( @@FETCH_STATUS = 0 ) 
BEGIN 
    PRINT '        Login - ' + @name + '    LoginIs - ' + @ServerRole 

    FETCH NEXT FROM sysAdmin_cursor INTO 
    @ServerRole , @name 

END 

CLOSE sysAdmin_cursor 
DEALLOCATE sysAdmin_cursor 

PRINT '' 

PRINT 'I. Name of Members in Serveradmin role.:' 

--DECLARE @serverAdmin table (ServerRole CHAR(20), name CHAR(50), memberid VARBINARY(128))
DECLARE @serverAdmin TABLE ( ServerRole char ( 20 ) , name sysname ) 
DECLARE @ServerRole1 char ( 20 ) , @name1 char ( 50 ) 

INSERT INTO @serverAdmin 
SELECT 
r.name , 
p.name AS MemberName 
FROM sys.server_principals r 
JOIN sys.server_role_members m ON r.principal_id = m.role_principal_id 
JOIN sys.server_principals p ON p.principal_id = m.member_principal_id 
WHERE ( r.type = 'R' ) 
AND 
( r.name = 'serveradmin' ) 

DECLARE serverAdmin_cursor CURSOR FOR 
SELECT 
ServerRole , 
name 
FROM @serverAdmin 

OPEN serverAdmin_cursor 

FETCH NEXT FROM serverAdmin_cursor INTO 
@ServerRole1 , @name1 

WHILE ( @@FETCH_STATUS = 0 ) 
BEGIN 
    PRINT '        Login - ' + @name1 + '    LoginIs - ' + @ServerRole1 

    FETCH NEXT FROM serverAdmin_cursor INTO 
    @ServerRole1 , @name1 

END 

CLOSE serverAdmin_cursor 
DEALLOCATE serverAdmin_cursor 

PRINT ' ' 

--Verify TEMP DB files.
PRINT 'J. Temp DB File Location.................:' 

DECLARE @filename nvarchar ( 520 ) 

DECLARE tempfile_cursor CURSOR FOR 
SELECT filename FROM sys.sysaltfiles WHERE name LIKE '%temp%' 

OPEN tempfile_cursor 

FETCH NEXT FROM tempfile_cursor INTO 
@filename 

WHILE ( @@FETCH_STATUS = 0 ) 
BEGIN 

    PRINT 'TEMP DB File......' + @filename 

    FETCH NEXT FROM tempfile_cursor INTO 
    @filename 
END 
CLOSE tempfile_cursor 
DEALLOCATE tempfile_cursor 

--Query to get LUN names from SQL server
PRINT ' ' 
PRINT 'K. MDF, LDF & NDF File Location..........:' 
DECLARE @Physical_Name nvarchar ( 520 ) 

DECLARE masterfile_cursor CURSOR FOR 
SELECT physical_name FROM sys.master_files WHERE physical_name NOT LIKE '%tempdb%' 

OPEN masterfile_cursor 

FETCH NEXT FROM masterfile_cursor INTO 
@Physical_Name 

WHILE ( @@FETCH_STATUS = 0 ) 
BEGIN 

    PRINT 'Physical Name......' + @Physical_Name 

    FETCH NEXT FROM masterfile_cursor INTO 
    @Physical_Name 
END 
CLOSE masterfile_cursor 
DEALLOCATE masterfile_cursor 

--use below query to  display the Minimun and Maximum memory in (MB)
PRINT ' ' 

DECLARE @Value sql_variant 
DECLARE @val_InDecimal decimal 
SELECT @Value = value FROM sys.configurations WHERE name = 'max server memory (MB)' ; 

SET @val_InDecimal = convert ( decimal , @Value ) / 1024 
PRINT 'L. Max server Memory(GB).................: ' + convert ( varchar ( 50 ) , @val_InDecimal ) 

SET @Value = NULL 
SET @val_InDecimal = NULL 
SELECT @Value = value FROM sys.configurations WHERE name = 'min server memory (MB)' ; 

SET @val_InDecimal = convert ( decimal , @Value ) / 1024 
PRINT 'M. Min server memory (GB)................: ' + convert ( varchar ( 50 ) , @val_InDecimal ) 

-- How to check Lock Pages In Memory is enabled
--You can use below simple technique to check whether lock pages in memory is enabled or not. 

CREATE TABLE #xp_cmdshell_output ( Output varchar ( 8000 ) ) ; 

-- The whoami command is run as the AD Account that SQL is running under
-- which is the account that needs to have lock pages in memory.
INSERT INTO #xp_cmdshell_output 

EXEC ( 'xp_cmdshell ''whoami /priv''' ) ; 

IF EXISTS 
( 
    SELECT * FROM #xp_cmdshell_output WHERE Output LIKE '%SeLockMemoryPrivilege%enabled%' 
) 
PRINT 'N. Lock Pages in Memory..................: Enabled' 
ELSE 
PRINT 'N. Lock Pages in Memory..................: Disabled' ; 

DROP TABLE #xp_cmdshell_output; 

PRINT char ( 13 ) 
PRINT char ( 13 ) 
PRINT char ( 13 ) 
PRINT 'Ignore the line below.' 

EXEC sp_configure 'xp_cmdshell' , 0 
RECONFIGURE