
--================================================================================================== Section 1: --====================================================================================
USE [master] 
GO 
IF db_id ( 'FnDBLogTest' ) IS NOT NULL 
BEGIN 
    DROP DATABASE FnDBLogTest; 
END 
GO 
CREATE DATABASE FnDBLogTest; 
GO 
----============== Setting the recovery model for database as FULL
ALTER DATABASE FnDBLogTest SET RECOVERY FULL; 
GO 
USE FnDBLogTest; 
GO 
IF object_id ( 'Example' , 'U' ) IS NOT NULL 
BEGIN 
    DROP TABLE Example; 
END 
GO 
----============== Creating a test table with some data for our demo
CREATE TABLE Example 
( 
    TheID int IDENTITY ( 1 , 1 ) , TheData varchar ( 1 ) 
) 
GO 
INSERT INTO Example 
VALUES ( 'A' ) , 
( 'B' ) , 
( 'C' ) , 
( 'D' ) 
GO 
----============== Taking a Full database backup to ensure that database is not 
----============== in pseudo Simple recovery model
BACKUP DATABASE FnDBLogTest 
TO DISK = 'D:\SQLTestLab\FnDBLogTest.BAK' WITH STATS = 10 , INIT 
GO 
----============== Truncating the test table
TRUNCATE TABLE Example 

--============== Retrieving the last TransactionID for Example
DECLARE @ObjectName sysname 
DECLARE @TransactionID nvarchar ( 500 ) 

SET @ObjectName = 'dbo.Example' 
-- Your schema qualified table name here
SET @TransactionID = 
( 
    SELECT TOP 1 [Transaction ID] FROM fn_dblog ( NULL , NULL ) WHERE AllocUnitName = @ObjectName 
    ORDER BY [Transaction ID] DESC 
) 
PRINT @TransactionID 

--============== Retrieving the UserName & Time when the table was truncated,
--============== based on the TransactionID
SELECT 
[Transaction Name] , 
suser_sname ( [Transaction SID] ) AS UserName , 
[Begin Time] , 
Operation , 
[Transaction ID] 
FROM fn_dblog ( NULL , NULL ) 
WHERE [Transaction ID] = '0000:000002e3' -- @TransactionID
AND [Transaction Name] LIKE 'TRUNCATE%' 
AND Operation = 'LOP_BEGIN_XACT' 

SET NOCOUNT ON; 
DECLARE @ObjectName sysname 
DECLARE @TransactionID nvarchar ( 500 ) 

SET @ObjectName = 'dbo.Example' 
-- Your schema qualified table name here

--============== Retrieving the last TransactionID for the table
SET @TransactionID = 
( 
    SELECT TOP 1 [Transaction ID] FROM fn_dblog ( NULL , NULL ) WHERE AllocUnitName = @ObjectName 
    ORDER BY [Transaction ID] DESC 
) 

PRINT 'Transaction Id : ' + @TransactionID 
--============== Retrieving the UserName & Time when the table was truncated, based on the TransactionID

SELECT 
@ObjectName AS ObjectName , 
[Transaction Name] , 
suser_sname ( [Transaction SID] ) AS UserName , 
[Begin Time] , 
Operation , 
[Transaction ID] 
FROM fn_dblog ( NULL , NULL ) 
WHERE [Transaction ID] = @TransactionID -- Transaction ID
AND [Transaction Name] LIKE 'TRUNCATE%' 
AND Operation = 'LOP_BEGIN_XACT' 

USE [master] 
GO 
CREATE DATABASE PITRTest ON ( NAME = 'PITRTest' , FILENAME = 'D:\SQLTestLab\PITRTest.mdf' ) 
log ON ( NAME = 'PITRTest_log' , FILENAME = 'D:\SQLTestLab\PITRTest_log.ldf' ) 
GO 
USE PITRTest 
GO 
----============== Setting the recovery model for database as FULL
ALTER DATABASE PITRTest SET RECOVERY FULL 
GO 
IF object_id ( 'Example' ) IS NOT NULL 
BEGIN 
    DROP TABLE Example 
END 
GO 
----============== Creating a test table with some data for our demo
CREATE TABLE Example 
( 
    ID int IDENTITY ( 1 , 1 ) , Data varchar ( 1 ) 
) 
GO 
INSERT INTO Example 
( Data ) 
VALUES ( 'A' ) , 
( 'B' ) , 
( 'C' ) , 
( 'D' ) , 
( 'E' ) 

----============== Taking a Full database backup
BACKUP DATABASE PITRTest TO DISK = 'D:\SQLTestLab\PITRTest_FULL_1.BAK' 
WITH STATS = 1 , INIT 
-- After the FULL BACKUP IS done , we INSERT few more rows INTO our test TABLE AND take the first T - log backup. 
INSERT INTO Example 
VALUES ( 'F' ) , 
( 'G' ) , 
( 'H' ) , 
( 'I' ) , 
( 'J' ) 
GO 
BACKUP log PITRTest TO DISK = 'D:\SQLTestLab\PITRTest_Log_1.trn' 
WITH STATS = 1 , INIT 
GO 

INSERT INTO Example VALUES ( 'K' ) , ( 'L' ) , ( 'M' ) , ( 'N' ) , ( 'O' ) , ( 'P' ) , ( 'Q' ) , ( 'R' ) , ( 'S' ) , ( 'T' ) 
GO 
--=============== Printing the time before performing the truncate
SELECT getdate ( ) AS TimeBeforeTruncate ---> 2014-04-18 02:20:37.277

GO 
--=============== Truncating the table
TRUNCATE TABLE Example 
GO 

BACKUP log PITRTest TO DISK = 'D:\SQLTestLab\PITRTest_Log_2.trn' WITH STATS = 1 , INIT 

RESTORE DATABASE PITRTest_2 FROM DISK = 'D:\SQLTestLab\PITRTest_FULL_1.BAK' 
WITH STATS = 1 , STANDBY = 'D:\SQLTestLab\PITRTestStandBy.tuf' , 
MOVE 'PITRTest' TO 'D:\SQLTestLab\PITRTest_2.mdf' , 
MOVE 'PITRTest_log' TO 'D:\SQLTestLab\PITRTest_2_log.ldf' 


SELECT * FROM 
PITRTest_2..Example --> 5 Rows 


 
RESTORE log PITRTest_2 FROM DISK = 'D:\SQLTestLab\PITRTest_Log_1.trn' 
WITH STATS = 1 , STANDBY = 'D:\SQLTestLab\PITRTestStandBy.tuf' 
GO 
SELECT * FROM PITRTest_2..Example -- 10 Rows

--=============== Stopping the restore at a point just before the truncate 

RESTORE log PITRTest_2 FROM DISK = 'D:\SQLTestLab\PITRTest_Log_2.trn' 
WITH STATS = 1 , STANDBY = 'D:\SQLTestLab\PITRTestStandBy.tuf' , 
STOPAT = '2014-04-18 02:20:37.277' 
GO 
SELECT * FROM PITRTest_2..Example -- All 20 Rows

--=============== Continuing the restore beyond the point where truncate was executed

RESTORE log PITRTest_2 FROM DISK = 'D:\SQLTestLab\PITRTest_Log_2.trn' 
WITH STATS = 1 , STANDBY = 'D:\SQLTestLab\PITRTestStandBy.tuf' , 
STOPAT = '2014-04-18 02:20:38.000' 
GO 
SELECT * FROM PITRTest_2..Example -- 0 Rows 
-- (Because we restored the log beyond the point where table was truncated.)